#!/bin/sh
set -eu

# version_greater A B returns whether A > B
version_greater() {
    [ "$(printf '%s\n' "$@" | sort -t '.' -n -k1,1 -k2,2 -k3,3 -k4,4 | head -n 1)" != "$1" ]
}

# Execute all executable files in a given directory in alphanumeric order
run_path() {
    hook_folder_path="/docker-entrypoint-hooks.d/$1"
    return_code=0

    if ! [ -d "${hook_folder_path}" ]; then
        echo "=> Skipping the folder \"${hook_folder_path}\", because it doesn't exist"
        return 0
    fi

    echo "=> Searching for scripts (*.sh) to run, located in the folder: ${hook_folder_path}"

    (
        find "${hook_folder_path}" -type f -maxdepth 1 -iname '*.sh' -print | sort | while read -r script_file_path; do
            if ! [ -x "${script_file_path}" ]; then
                echo "==> The script \"${script_file_path}\" was skipped, because it didn't have the executable flag"
                continue
            fi

            echo "==> Running the script (cwd: $(pwd)): \"${script_file_path}\""

            "${script_file_path}" || return_code="$?"

            if [ "${return_code}" -ne "0" ]; then
                echo "==> Failed at executing \"${script_file_path}\". Exit code: ${return_code}"
                exit 1
            fi

            echo "==> Finished the script: \"${script_file_path}\""
        done
    )
}

version_check() {
    installed_version="0.0.0"

    if [ -f "$DOLI_DATA/.latest_version" ]; then
        installed_version="$(cat "$DOLI_DATA/.latest_version")"
    fi

    if version_greater "$installed_version" "$DOLI_VERSION"; then
        echo "Can't start Dolibarr because the version of the data ($installed_version) is higher than the docker image version ($DOLI_VERSION) and downgrading is not supported. Are you sure you have pulled the newest image version ?"
        exit 1
    fi

    if version_greater "$DOLI_VERSION" "$installed_version"; then
        echo "Initializing Dolibarr $DOLI_VERSION ..."

        echo "Waiting for database..."
        while ! nc -z "$DOLI_DB_HOST" "$DOLI_DB_PORT"; do sleep 1; done

        if [ "$installed_version" = "0.0.0" ]; then
            install
        else
            upgrade
        fi

        echo "$DOLI_VERSION" > "$DOLI_DATA/.latest_version"
        chmod 440 "$DOLI_HTDOCS/conf/conf.php"

        echo "Initializing finished"
    fi
}

install() {
    echo "New Dolibarr instance"

    if [ -n "$DOLI_LANG" ] && [ -n "$DOLI_URL_ROOT" ] && [ -n "$DOLI_ADMIN_LOGIN" ] && [ -n "$DOLI_ADMIN_PASSWORD" ] && [ -n "$DOLI_DB_TYPE" ] && [ -n "$DOLI_DB_HOST" ] && [ -n "$DOLI_DB_PORT" ] && [ -n "$DOLI_DB_USER" ] && [ -n "$DOLI_DB_PASSWORD" ] && [ -n "$DOLI_DB_NAME" ]; then
        echo "Installing with $DOLI_DB_TYPE database"

        run_path pre-installation

        echo "Starting Dolibarr installation"

        cd "$DOLI_HTDOCS/install"
        echo "Dolibarr installer step 2..."
        php step2.php set "$DOLI_LANG" > /tmp/dolibarr_install_step2.log 2>&1
        echo "Dolibarr installer step 4..."
        php step4.php "$DOLI_LANG" > /tmp/dolibarr_install_step4.log 2>&1
        echo "Dolibarr installer step 5..."
        php step5.php 0.0.0 0.0.0 fr_FR set "$DOLI_ADMIN_LOGIN" "$DOLI_ADMIN_PASSWORD" "$DOLI_ADMIN_PASSWORD" > /tmp/dolibarr_install_step5.log 2>&1
        cd "$DOLI_WWW"

        run_path post-installation
    else
        echo "Please run the web-based installer on first connect!"
    fi
}

upgrade() {
    echo "Upgrading Dolibarr from $installed_version ..."

    run_path pre-upgrade

    cd "$DOLI_HTDOCS/install"

    from="0.0.0"
    to="0.0.0"
    while true; do
        version="$(php "$DOLI_WWW/scripts/migration-path.php")"
        old_from="$from"
        from="$(echo "$version" | sed -E 's/^([^;]+);[^;]+$/\1/')"
        old_to="$to"
        to="$(echo "$version" | sed -E 's/^[^;]+;([^;]+)$/\1/')"

        if [ "$old_from" = "$from" ] && [ "$old_to" = "$to" ]; then
            break;
        fi

        echo "$from => $to"
        php upgrade.php "$$from" "$to" >> /tmp/upgrade.log
        php upgrade2.php "$from" "$to" >> /tmp/upgrade.log
    done

    php step5.php "$from" "$to" >> /tmp/upgrade.log

    run_path post-upgrade
}

main() {
    # If another process is migrating the database, wait for it to be done, then escape initalization.
    (
        if ! flock -n 9; then
            # If we couldn't get it immediately, show a message, then wait for real
            echo "Another process is initializing Dolibarr. Waiting..."
            flock 9
        fi

        if [ "$DOLI_SKIP_VERSION_CHECK" = "false" ]; then
            version_check
        fi
    ) 9> "$DOLI_ROOT_ALT/.dolibarr-init-sync.lock"

    run_path before-starting
}

if [ "$1" = "docker-php-entrypoint" ]; then
    main
fi

exec "$@"
